import importlib
import logging
import os
import time
import traceback
from time import sleep
from kthread import KThread

from bbcloud_autotest_sdk.airtest_babybus.report.report import LogToHtml
from bbcloud_autotest_sdk.airtest_babybus.utils.retry import retries
from bbcloud_autotest_sdk.device_platform.babybus_android_utils import BabybusAndroidUtils
from bbcloud_autotest_sdk.airtest_babybus.core.android.adb import ADB
from bbcloud_autotest_sdk.airtest_babybus.core.api import connect_device, auto_setup
from bbcloud_autotest_sdk.airtest_babybus.utils.apkparser import APK
import bbcloud_python_sdk as utils

from bbcloud_autotest_sdk.poco_babybus.drivers.android.uiautomation import AndroidUiautomationPoco
from bbcloud_autotest_sdk.poco_babybus.drivers.std import StdPoco
from bbcloud_autotest_sdk.public.perform import _perform_get_app_pid, _perform_get_m3_flow, _perform_get_top_total_cpu, \
    _perform_get_all_mem, _perform_get_app_mem, _perform_get_battery


class AndroidEngine():
    # 安卓Poco对象
    android_poco_obj: AndroidUiautomationPoco = None
    # 当前标准的Poco对象
    app_poco_obj: StdPoco = None
    # 当前设备系统对象实例
    system_obj: BabybusAndroidUtils = None
    # 当前测试程序类型
    airtest_script_type: int = 0
    # 当前测试app的apk id
    app_id: str = ""
    # app包的本地路径
    app_path = ''
    # 当前测试用例仓库id
    cases_project_id: int = None
    # 当前设备的adb id
    adb_id: str = ""
    # 当前任务的主id
    number: str = ""
    # 全局结果存放路径，最后会被minio上传
    result_path: str = ""
    result_url: str = ""
    # 全局与log服务器进行交互的socket
    server_socket = ""
    # 全局与log服务器写log方法
    server_log_func = None
    # 全局com对象，该类实现兼容方法
    com_obj = None
    # 测试脚本列表
    script_name = None
    # 全局截图函数
    screen_shot_func = None
    # task_handle对象
    task_handle_obj = None
    # 当前手机品牌
    brand: str = ""
    # 当前手机系统
    version: str = ""
    # 是否是通用脚本
    is_common_script: bool = False
    # 一些其他的变量
    context: dict = {}
    # oppo_click是否停止
    oppo_click_stop = False
    # 自动截图
    auto_shot = False
    # 日志目录
    report_dir = None

    def __init__(self, adb_id, cases_project_id, app_path,
                 airtest_script_type, number, report_dir):
        self.adb_id = adb_id
        self.cases_project_id = cases_project_id
        self.airtest_script_type = airtest_script_type
        self.app_path = app_path
        self.number = number
        self.report_dir = report_dir
        if not os.path.exists(report_dir):
            os.mkdir(report_dir)
        self.click_node_thread: KThread = None
        self.script_run_err_info = None
        self.stop_click = False
        self._init_system_obj()

    def init_android_poco(self):
        """
        初始化安卓poco
        """
        use_airtest_input = True
        self.system_obj.shell("am start -n com.netease.open.pocoservice/.TestActivity")
        self.android_poco_obj = AndroidUiautomationPoco(screenshot_each_action=False,
                                                        use_airtest_input=use_airtest_input)

    # def init_app_poco(self):
    #     poco = StdPoco(devices=self.system_obj)
    #     poco.get_screen_size()
    #     self.app_poco_obj = poco
    #     if 'OPPO' == self.brand:
    #         self.oppo_click_stop = True

    def _init_system_obj(self):
        """
        初始化安卓对象，该对象实现大量安卓设备的方法
        """
        model = ADB(self.adb_id).getprop("ro.product.brand").upper()
        self.system_obj = connect_device("Android:///%s" % self.adb_id)
        # self.system_obj.rotation_watcher.start()
        auto_setup(
            __file__,
            logdir="%s/log" % self.report_dir,
            devices=["Android:///%s" % self.adb_id]
        )
        self.brand = self.system_obj.getprop("ro.product.brand").upper()
        self.version = self.system_obj.getprop("ro.build.version.release")

    @staticmethod
    def _get_image_name():
        """
        检索目标目录，返回一个可用的文件名
        :return:
        """
        # with Lock(cache, "get_screen_capture_name", 2):
        #     index = 0
        #     target_path = os.path.join(self.result_path, "%d.png" % index)
        #     while os.path.exists(target_path):
        #         index += 1
        #         target_path = os.path.join(self.result_path, "%d.png" % index)
        #     return target_path

    @staticmethod
    def _img_write_text(path, desc):
        """
        使用cv2在图片上写字
        :param path:
        :param desc:
        """
        # img = cv2.imread(path)
        # sp = img.shape
        # height = sp[0]
        #
        # def cv2ImgAddText(img, text, left, top, textColor=(255, 0, 0), textSize=126):
        #     if isinstance(img, numpy.ndarray):  # 判断是否OpenCV图片类型
        #         img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
        #     # 创建一个可以在给定图像上绘图的对象
        #     draw = ImageDraw.Draw(img)
        #     # 字体的格式
        #     fontStyle = ImageFont.truetype(os.path.join(FONT_DIR, "font.ttc"), textSize,
        #                                    encoding="utf-8")
        #     # 绘制文本
        #     draw.text((left, top), text, textColor, font=fontStyle)
        #     # 转换回OpenCV格式
        #     return cv2.cvtColor(numpy.asarray(img), cv2.COLOR_RGB2BGR)
        #
        # # img = cv2.putText(img, desc, (50, int(height / 5)), cv2.FONT_HERSHEY_COMPLEX, 3, (0, 0, 255), 3)
        # img = cv2ImgAddText(img, desc, 50, int(height / 5))
        # cv2.imwrite(path, img)

    # def _export_func(self):
    #     """
    #     根据是否调试导出相关函数
    #     :return:
    #     """
    #     if IS_SKIP:
    #         return [
    #             self.run_case,
    #             self.stop_case,
    #         ]
    #     else:
    #         return [
    #             self.start_click_node,
    #             self.uninstall_other_apk,
    #             self.set_device_prop,
    #             self.start_recorder,
    #             self.install_apk,
    #             self.start_app,
    #             self._wait_for_app_run,
    #             self._init_logcat,
    #             self.stop_click_node,
    #             self.start_perform_test_thread,
    #             self.start_auto_screen_shot_thread,
    #             self.run_case,
    #             self.stop_case,
    #         ]

    def screen_capture(self, desc=""):
        """
        截图
        :param desc:
        :return:
        """

        str_path = self._get_image_name()
        try:
            self.system_obj.snapshot(str_path)
            self._img_write_text(str_path, desc)
        except Exception as e:
            e = traceback.format_exc()
            print(e)
            pass
        url = os.path.join(self.result_url, str_path.split('/')[-1])
        return url

    def snap_shot(self, action_en):
        """
        截图并且存入日志
        :param action_en:
        """
        try:
            url = self.screen_capture(action_en)
            # Report.get_instance().add_screen_capture(url)
        except:
            pass

    def uninstall_other_apk(self):
        """
        开始卸载其他宝宝巴士产品，防止影响测试。
        :return:
        """

        if "HUAWEI" in self.brand:
            try:
                self.system_obj.shell("uninstall --user 0 com.huawei.motionservice")
            except:
                pass

        for app in self.system_obj.list_app():
            if ('babybus' in app) or ('lianyong' in app) or ('kid58' in app) or ('sinyee' in app) or ('yw' in app) or (
                    'ailiyong' in app) or ('kidvideo' in app):
                if 'airtest.helper' in app:
                    continue
                print("uninstall apk:%s" % app)
                try:
                    self.system_obj.uninstall_app(app)
                except TimeoutError:
                    raise
                except:
                    pass

    def install_apk(self):

        apk_obj = APK(self.app_path)
        self.app_id = apk_obj.get_package()
        # 将屏幕点亮
        try:
            if not self.system_obj.is_screenon():
                self.system_obj.keyevent("power")
            self.light_screen()
        except:
            pass

        self.uninstall_apk()

        # 开始安装apk

        # 真实安装apk函数实现
        self.install_imp(self.app_path)

        # if self.brand == "OPPO":
        #     sleep(40)

        if self.adb_id in ['5TYLCI4SMBTWDQY5', '785ab21a', '6b91dee8', '842dda7f', '1729b4be']:
            sleep(100)

        # 检查APK是否安装成功
        if self.app_id not in self.system_obj.list_app():
            raise RuntimeError("安装apk时失败")

    def start_click_node(self):
        """
        该函数会启动线程持续点击弹出的各种弹窗，以及安装时要求输入的东西，配合安卓层服务一起进行
        """
        while not self.stop_click:
            self.click_node()
            sleep(1)
        # def inner():
        #     while not self.stop_click:
        #         self.click_node()
        #         sleep(1)
        #
        # self.click_node_thread = KThread(target=inner)
        # self.click_node_thread.setName("click node thread")
        # self.click_node_thread.start()

        # def clean_up():
        #     if self.click_node_thread.is_alive():
        #         self.click_node_thread.kill()

    def stop_click_node(self):
        try:
            self.click_node_thread.kill()
        except:
            pass
        self.stop_click = True

    def uninstall_apk(self):
        """
        该方法，用于卸载安卓层相同的旧apk，需要在cocos里进行重写，删除所有其他2d app
        """
        try:
            self.system_obj.uninstall_app(self.app_id)
        except:
            pass

    def start_app(self):
        """
        启动app函数
        """
        self.system_obj.start_app(self.app_id)
        is_activity = False
        while not is_activity:
            logging.info("正在等待应用：%s 启动..." % self.app_id)
            is_activity = self.app_id in self.system_obj.get_top_activity()
            sleep(3)
        if 'com.sinyee.babybus.chants' == self.app_id:
            sleep(20)

        logging.info("应用：%s 已启动!" % self.app_id)

    def start_perform_test_thread(self):
        """
        开始性能测试的线程
        """

        # if IS_DEBUG or self.sub_task_num.startswith("TSUB"):
        #     return

        def inner():
            while True:
                try:
                    self.get_perform()
                except:
                    pass
                sleep(1)

        handle = KThread(target=inner)
        handle.setName("perform test thread")
        handle.start()

    def start_auto_screen_shot(self):
        """
        该函数用于定时自动截图
        """

        while self.auto_shot:
            sleep(2 * 60)
            try:
                cur_time = "cur_time_%s" % time.strftime('%H:%M:%S ', time.localtime(time.time()))
                self.screen_shot_func(cur_time)
            except:
                pass

    def start_recorder(self):
        try:
            self.system_obj.recorder.stop_recording(is_interrupted=False)
        except:
            pass

        self.system_obj.recorder.start_recording(max_time=2 * 60 * 60)

    def stop_recorder(self):
        self.system_obj.recorder.stop_recording(is_interrupted=False,
                                                output=os.path.join(self.report_dir, 'screen.mp4'))

    def run_case(self):
        """
        启动测试用例脚本
        """

        # target_case = importlib.import_module(relate_dir_path.replace(os.path.sep, '.') + "." + case)
        # target_case.case_bb().main()

    def stop_case(self):
        """
        结束时调用该函数
        """
        pass

    def finish_task(self):
        """
        结束测试任务的时候调用
        """
        try:
            pass
        except:
            pass

    def set_device_prop(self):
        # 开启设备上宝宝巴士产品的测试模式、自动化模式
        self.system_obj.shell(r"setprop debug.babybus.device all")
        self.system_obj.shell(r"setprop debug.babybus.app all")
        self.system_obj.shell(r"setprop debug.babybus.auto all")
        # 该设置在部分手机上不会返回内容，导致函数报错。
        try:
            self.system_obj.shell(r"svc power stayon true")
        except:
            pass
        # self.system_obj.shell(r"svc wifi disable")
        # self.system_obj.shell(r"svc wifi enable")
        self.system_obj.shell(
            "content insert --uri content://settings/system --bind name:s:accelerometer_rotation --bind value:i:0")

    def get_perform(self):
        pid = _perform_get_app_pid()
        flow_data = _perform_get_m3_flow(pid)
        usage = _perform_get_top_total_cpu()
        result_dict = {
            'system_cpu': usage,
            'test_cpu': usage,
            'system_memory': _perform_get_all_mem(
                adb_path=self.system_obj.adb.adb_path,
                adb_id=self.adb_id,
            ),
            'test_memory': _perform_get_app_mem(
                adb_path=self.system_obj.adb.adb_path,
                adb_id=self.adb_id,
                app_id=self.app_id
            ),
            'battery': _perform_get_battery(
                adb_path=self.system_obj.adb.adb_path,
                adb_id=self.adb_id),
            'system_flow': round(flow_data['system_flow'], 2),
            'system_up_flow': round(flow_data['system_up_flow'], 2),
            'system_down_flow': round(flow_data['system_down_flow'], 2),
            'number': self.number,
            'system_fps': '',
            'system_time': time.time()
        }

    def get_app_poco(self):
        if self.app_poco_obj:
            return self.app_poco_obj
        self.app_poco_obj = StdPoco()
        return self.app_poco_obj

    def install_imp(self, target_dir):
        # 附带-g选项，该选项安装后就拥有所有权限
        def install_with_permission():
            self.system_obj.install_app(target_dir, replace=True, install_options=['-g', '-t', '-d'])

        if self.brand not in ["SAMSUNG", "OPPO", "VIVO"]:
            try:
                install_with_permission()
                return
            except Exception as e:
                e = traceback.format_exc()
                print(e)

        def install_without_permission():
            self.system_obj.install_app(target_dir, replace=True, install_options=['-t', '-d'])

        for _ in range(3):
            try:
                install_without_permission()
                return
            except Exception as e:
                e = traceback.format_exc()
                print(e)

    def poco_init(self):

        poco = StdPoco(devices=self.system_obj)
        poco.get_screen_size()
        self.app_poco_obj = poco

        if 'OPPO' == self.brand:
            self.oppo_click_stop = True

    def light_screen(self):
        self.system_obj.wake()
        self.system_obj.shell("input swipe 342 1129 342 352")
        self.system_obj.shell("input swipe 99 907 551 907")
        self.system_obj.wake()

    def click_node(self):
        # 各类设备权限弹窗处理
        self.allow_box()
        self.password_input()
        if 'OPPO' == self.brand:
            pass
            # file_path = os.path.join("/tmp", "ocr", self.adb_id + ".jpg")
            # snap_data = self.system_obj.snapshot(file_path)

    def password_input(self):
        """
        密码输入处理
        :return:
        """
        pocoAndroid = self.android_poco_obj.freeze()
        if pocoAndroid("com.bbk.account:id/dialog_pwd").exists():  # vivo设备输入密码
            self.system_obj.shell("input text 'babybus123'")

        if pocoAndroid("com.coloros.safecenter:id/et_login_passwd_edit").exists() \
                and pocoAndroid("com.coloros.safecenter:id/et_login_passwd_edit").attr('visible'):  # oppo设备输入密码
            self.system_obj.shell("input text 'babybus123'")

    def allow_box(self):
        """
        权限弹窗
        :return:
        """
        pocoAndroid = self.android_poco_obj.freeze()

        # click_name 和 click_text 所指的按钮多数都是重复的，个别应用内的按钮是图片，所以无法识别按钮文字，只能用按钮的poco名称
        click_name = [
            "com.android.packageinstaller:id/permission_allow_button",  # 矩阵：权限-始终允许按钮
            self.app_id + ":id/tv_have_read",  # 矩阵：隐私协议-同意并使用按钮
            self.app_id + ":id/privace_dialog_sure",  # 平台：隐私协议-同意按钮
            self.app_id + ":id/permission_dialog_sure",  # 平台：权限-同意按钮
            # self.app_id + ":id/tv_age_1",  # 平台：宝宝巴士-年龄选择框 0-3岁
            self.app_id + ":id/img_baby_close",
            "com.huawei.systemmanager:id/btn_allow",
            "com.android.packageinstaller:id/ignore_tv",
            "com.android.packageinstaller:id/done_button"
        ]

        click_text = {
            "继续安装旧版本": [0.5, 0.5],
            "继续安装": [0.5, 0.5],
            "允许": [0.5, 0.5],
            "确定": [0.5, 0.5],
            "好": [0.5, 0.5],
            "安装": [0.5, 0.5],
            "传输文件": [0.5, 0.5],
            "始终允许": [0.5, 0.5],
            "继续": [0.5, 0.5],
            "完成": [0.5, 0.5],
            "同意": [0.5, 0.5],
            "稍后": [0.5, 0.5],
            "稍后提醒": [0.5, 0.5],
            "是，访问数据": [0.5, 0.5],
            "关闭应用": [0.5, 0.5],
            "是": [0.5, 0.5],
            "打开": [0.5, 0.5],
            "我已知问题严重性。无视风险安装": [0.8, 0.5],
            "无视风险安装": [0.5, 0.5],
            "我知道了": [0.5, 0.5],
            "总是允许": [0.5, 0.5],
        }

        for node in click_name:
            if pocoAndroid(node).exists():
                # print('name' + node)
                pocoAndroid(node).click()

        for node in click_text:
            if pocoAndroid(text=node).exists():
                pocoAndroid(text=node).click(click_text[node])

        # 开启取消同时出现的时候点取消
        if pocoAndroid(text="开启").exists() and pocoAndroid(text="取消").exists():
            pocoAndroid(text="取消").click()

        if pocoAndroid(text="取消").exists() and pocoAndroid(text="切换").exists():
            pocoAndroid(text="取消").click()

    def report_to_html(self, script_root):
        rp = LogToHtml(
            static_root='https://v3-file-bbcloud.babybus.com/autotest/airtest_report/static',
            script_root=script_root,
            lang='zh',
            export_dir=self.report_dir,
            plugins=['bbcloud_autotest_sdk.poco_babybus.utils.airtest.report'])

        rp.report(output_file='index.html')
