import random
import re

from bbcloud_autotest_sdk.public.cmd.monkey_config import config_map_click_num
from time import sleep

from kthread import KThread

from bbcloud_autotest_sdk.poco_babybus.drivers.std import StdPoco


class MonkeyObj:
    def __init__(self, engine):
        self.engine = engine
        self.poco = engine.app_poco_obj
        self.scene = None
        self.scene_code = ""
        self.mul_touch_num = self.get_mul_touch_num()
        if self.poco:
            try:
                self.w, self.h = self.poco.get_screen_size()
            except:
                pass
        self.has_remove_ad = False

    def start_continue_remove_ad(self):
        # 判断是否去广告
        if self.has_remove_ad:
            return
        self.has_remove_ad = True
        self.poco.agent.eval_code(
            "cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() NV.removeAd() end, 0.1, false)")

    def _search(self, ui, name, result, key="name"):
        if not ui.get("children"):
            return
        for node in ui['children']:
            if name in node['payload'].get(key, ""):
                if node['payload']['type'] == "Sprite" or node['payload']['type'] == "Scale9Sprite":
                    result.append(node['name'])
            if node.get("children", False):
                self._search(node, name, result, key=key)

    def search(self, name, key="name"):
        ui = self.poco.agent.hierarchy.dump()
        result = []
        for item in ui['children']:
            self._search(item, name, result, key)
        result.reverse()
        return result

    def eval_code(self, code_exp):
        if not isinstance(self.poco, StdPoco):
            raise RuntimeError('only imp cocos2d poco!')
        result = self.poco.agent.eval_code(code_exp)
        return result

    def is_app_running(self):
        app_name = self.engine.apk_id
        try:
            return app_name in self.engine.system_obj.get_top_activity()
        except:
            self.engine.system_obj.wake()
            sleep(1)
            return app_name in self.engine.system_obj.get_top_activity()

    @staticmethod
    def in_pos(pos):
        for i in range(0, 1):
            if not (0 <= pos[i] >= 1):
                return False
            return True

    def check_scene(self):
        ui_tree = self.poco.agent.hierarchy.dump()
        return ui_tree['name'] == self.scene

    def eval_enter_scene(self):
        self.eval_code(self.scene_code)

    def check_now(self):
        # 这里并没有使用到poco，所以单独线程执行
        if not self.is_app_running():
            self.engine.system_obj.start_app(self.engine.apk_id)

    @staticmethod
    def get_scene(scene):
        p = re.compile('.+?"(.+?)"')
        scene_name = p.findall(scene)
        scenen_ = scene_name[0].capitalize()
        if not 'Scene' in scenen_:
            scenen_ += "Scene"
        return scenen_

    def scene_monkey(self, scene, count=300, type=0):
        '''
        场景monkey
        :param scene: 场景名称
        :param count: monkey次数
        :param type: 0-点击+滑动，1-点击，2-滑动
        :return:
        '''
        self.start_continue_remove_ad()
        self.scene = self.get_scene(scene)
        self.scene_code = scene

        def thread_checker():
            while True:
                self.check_now()
                sleep(1)

        handler_thread_checker = KThread(target=thread_checker)
        handler_thread_checker.start()

        if not self.check_scene():
            self.eval_enter_scene()

        for _ in range(count):
            self.click_event(type)

        try:
            handler_thread_checker.kill()
        except:
            pass

    def select_pos(self):
        # 当屏幕比例 大于1335/750=1.78 以高度来划分16：9的区域，反之用宽度。

        if self.w / self.h > 1335 / 750:  # 以高度（h）固定

            # 无论屏幕多大，游戏程序只在16：9的范围内填充游戏内容
            # 超出或小于 16：9的设备 需要扣除左右两边一定的距离 padding
            w_padding = int((self.w - self.h * 16 / 9) / 2)
            if w_padding < 0:
                w_padding = 0
            # 还需要加上返回键位置 返回键基础130*130
            left_w_padding = w_padding + int(self.h / 540 * 130)
            top_h_padding = int(self.h / 540 * 130)

        else:
            h_padding = int((self.h - self.w / 16 * 9) / 2)
            if h_padding < 0:
                h_padding = 0
            # 还需要扣除返回键位置 返回键基础130*130
            left_w_padding = int(self.w / 960 * 130)
            top_h_padding = h_padding + int(self.h / 540 * 130)

        pos = [0, 0]
        # 区域判断（抠去返回键位置）
        while 0 <= pos[0] <= left_w_padding and 0 <= pos[1] <= top_h_padding:
            pos = [random.randint(100, self.w - 100), random.randint(100, self.h - 100)]

        # 转换成屏幕上点击的坐标值
        pos = self.engine.system_obj._touch_point_by_orientation(pos)

        return pos

    def click_event(self, type):
        array_point = []
        num = self.mul_touch_num
        for _ in range(num):
            array_point.append(self.select_pos())
        if type == 2:
            # 滑动
            self.engine.system_obj.minitouch.swipe(self.select_pos(), self.select_pos(), duration=0.01)
        elif type == 1:
            # 点击
            self.engine.system_obj.minitouch.touch_array(array_point)
        else:
            # 点击+滑动
            self.engine.system_obj.minitouch.touch_array(array_point)
            self.engine.system_obj.minitouch.swipe(self.select_pos(), self.select_pos(), duration=0.01)

    def node_click(self, poco_name, area=None, times=300, type=0):
        """
        monkey点击某个元素，参数节点name
        :param poco_name:节点的名字
        :param area: 范围x1,x2,y1,y2
        :param times: 点击的次数
        :param type: 0-点击+滑动，1-点击，2-滑动
        :return:
        """
        if area is None:
            area = [0, 1, 0, 1]

        poco = self.poco.freeze()

        def random_pos():
            focus = [random.randint(area[0] * 10, area[1] * 10) / 10, random.randint(area[2] * 10, area[3] * 10) / 10]
            pos = poco(poco_name).get_position(focus)
            pos = [pos[0] * self.w, pos[1] * self.h]
            pos = self.engine.system_obj._touch_point_by_orientation(pos)
            return pos

        num = self.mul_touch_num
        if type == 2:
            for _ in range(times):
                self.engine.system_obj.minitouch.swipe(random_pos(), random_pos(), duration=0.01)
        elif type == 1:
            for _ in range(times):
                array_point = [random_pos() for _ in range(0, num)]
                self.engine.system_obj.minitouch.touch_array(array_point)
        else:
            for _ in range(times):
                array_point = [random_pos() for _ in range(0, num)]
                self.engine.system_obj.minitouch.touch_array(array_point)
                self.engine.system_obj.minitouch.swipe(random_pos(), random_pos(), duration=0.01)

    def get_mul_touch_num(self):
        return config_map_click_num.get(self.engine.adb_id, 1)
