# coding=utf-8
import math
import random
import time

from bbcloud_autotest_sdk.poco_babybus.drivers.std import StdPoco
from bbcloud_autotest_sdk.public.cmd.monkey import MonkeyObj
from bbcloud_autotest_sdk.public.cmd.time import TimeObj


class act:
    poco: StdPoco

    def __init__(self, engine):
        self.monkey = MonkeyObj(engine)
        self.time = TimeObj()
        self.android_poco_obj = engine.android_poco_obj
        self.app_poco = engine.get_app_poco()
        self.poco = self.app_poco

    def scene_check(self, scene):
        ui_tree = self.app_poco.agent.hierarchy.dump()
        if 'Scene' not in scene:
            scene += "Scene"
        if ui_tree['name'] != scene:
            raise Exception("运行测试用例时，发现与断言的场景不同，目标场景%s" % scene)

    def _search(self, ui, name, result, key="name", result_key="name"):
        if not ui.get("children"):
            return
        for node in ui['children']:
            # print(node)
            if name in node['payload'].get(key, ""):
                if node['payload']['type'] == "Sprite" or node['payload']['type'] == "Scale9Sprite" or node['payload'][
                    'type'].startswith("android") or node['payload']['type'] == "GameObject" or node['payload'][
                    'type'] == "Image" or node['payload']['type'] == "Button":
                    if result_key == "name":
                        result.append(node['name'])
                    else:
                        result.append(node['payload'][result_key])

            if node.get("children", False):
                self._search(node, name, result, key=key, result_key=result_key)

    def search(self, name, key="name", result_key="name", poco=None):
        if not poco:
            poco = self.app_poco

        ui = poco.agent.hierarchy.dump()
        result = []
        for item in ui['children']:
            self._search(item, name, result, key, result_key)
        # 这里很关键，考虑到搜索二叉树的结构，直接倒序整个列表，即可排序z高度
        result.reverse()
        return result

    def deep_order_nodes(self, nodes, asc=True):
        """
        对节点进行深度排序，返回排序过的节点名字
        :param nodes: 节点列表
        :param asc: 是否升序，默认升序
        """
        node_infos = []
        for node in nodes:
            node_info = {"name": node, "zOrders": self.app_poco(node).attr("zOrders")}
            node_infos.append(node_info)
        list.sort(node_infos, key=lambda x: x['zOrders']['local'], reverse=not asc)
        return [x['name'] for x in node_infos]

    def compare_pos(self, node_a, node_b):
        posA = self.app_poco(node_a).get_position("center")
        posB = self.app_poco(node_b).get_position("center")
        x1 = posA[0]
        x2 = posB[0]
        y1 = posA[1]
        y2 = posB[1]
        x = x2 - x1
        y = y2 - y1
        return math.sqrt((x * x) + (y * y))

    def search_drag_to(self, node_a, node_b, side_pos=None, drag_speed=0.3, drag_dis_time=0.7, key="name",
                       random_focus=False, check_dis=False, check_dis_value=0.1):
        poco = self.app_poco
        leafs = self.search(node_a, key)
        bottom = self.search(node_b, key)
        if leafs is None:
            return None
        for node in leafs:
            for bot in bottom:
                try:
                    if self.compare_pos(node, bot) <= check_dis_value and check_dis:
                        continue
                    if random_focus:
                        poco(node).drag_to(poco(bot).get_position(focus=self.random_focus()), drag_speed)
                    else:
                        poco(node).drag_to(poco(bot), drag_speed)
                    time.sleep(drag_dis_time)
                except:
                    pass
            if side_pos:
                poco(node).drag_to(side_pos, drag_speed)
            time.sleep(drag_dis_time)

    def btn_back(self):
        name = self.search("g/button/btn_back", "imgName_")
        if len(name) == 0:
            raise Exception("没有找到返回按钮!")
        self.app_poco(name[0]).click()

    @staticmethod
    def random_focus():
        return [random.randint(1, 100) / 100, random.randint(1, 100) / 100]

    def wait_click(self, name):
        self.app_poco(name).wait_for_appearance()
        time.sleep(2)
        self.app_poco(name).click()

    def wait_drag(self, name, target, dis=2):
        self.app_poco(name).wait_for_appearance()
        time.sleep(dis)
        self.app_poco(name).drag_to(self.app_poco(target), 0.5)

    def if_is_in_pos(self, name):
        pos = self.app_poco(name).get_position()
        if pos[0] <= 1 and pos[1] >= 0:
            if 1 >= pos[1] >= 0:
                return True
        return False

    def wait_in_pos(self, name):
        while not self.if_is_in_pos(name):
            pass

    def wait_in_screen_click(self, name):
        try:
            self.app_poco(name).click()
        except Exception as e:
            if "position" in str(e):
                self.wait_in_screen_click(name)

    def eval_code(self, code_exp):
        if not isinstance(self.app_poco, StdPoco):
            raise RuntimeError('only imp cocos2d poco!')
        result = self.app_poco.agent.eval_code(code_exp)
        return result

    def and_click(self, text):
        '''
        Android层，获取UI text坐标点击
        :param text:
        :return:
        '''
        pos_in_percentage = self.search(text, key="text", result_key='pos', poco=self.android_poco_obj)
        ret = self.app_poco.click(pos_in_percentage[0])
        return ret
