import json
import threading
# Const
# 设置HTTP接入域名（此处以公共云生产环境为例）
from time import sleep

from mq_http_sdk.mq_client import MQClient
from mq_http_sdk.mq_consumer import MQConsumer, Message
from mq_http_sdk.mq_exception import MQExceptionBase
from mq_http_sdk.mq_producer import MQProducer, TopicMessage

from global_var.conf import endpoint, accessKey, secretKey
from global_var.runtime import GB


class RocketMQ:
    consumer: MQConsumer
    mq_client: MQClient
    _instance_lock = threading.Lock()

    def __init__(self):
        self.mq_client = None
        # client生成的consumer 根据topic不同生成的表
        self.client_consumer_map = {}
        # consumer根据tag 不同生成的表
        self.consumer_tag_map = {}
        # producer根据tag 同上
        self.producer_tag_map = {}

    def _connect(self):
        self.mq_client = MQClient(
            # 设置HTTP接入域名（此处以公共云生产环境为例）
            endpoint,
            # AccessKey 阿里云身份验证，在阿里云服务器管理控制台创建
            accessKey,
            # SecretKey 阿里云身份验证，在阿里云服务器管理控制台创建
            secretKey
        )

    def _get_consumer(self, mq, tag: str) -> MQConsumer:
        with RocketMQ._instance_lock:
            token = f"{mq.instanceId}-{mq.topic}-{mq.groupId}"
            if token in self.consumer_tag_map:
                return self.consumer_tag_map[token]
            else:
                consumer = self.mq_client.get_consumer(mq.instanceId, mq.topic, mq.groupId, tag)
                self.consumer_tag_map[token] = consumer
                return consumer

    def _get_producer(self, mq) -> MQProducer:
        with RocketMQ._instance_lock:
            token = f"{mq.instanceId}-{mq.topic}-{mq.groupId}"
            if token in self.producer_tag_map:
                return self.producer_tag_map[token]
            else:
                producer = self.mq_client.get_producer(mq.instanceId, mq.topic)
                self.producer_tag_map[token] = producer
                return producer

    def init(self):
        self._connect()

    @classmethod
    def get_instance(cls, *args, **kwargs):
        """

        :rtype: RocketMQ
        """
        with RocketMQ._instance_lock:
            if not hasattr(RocketMQ, "_instance"):
                RocketMQ._instance = RocketMQ(*args, **kwargs)
        return RocketMQ._instance

    def send_message(self, mq, data_dict: dict, tag: str):
        msg = TopicMessage(
            # 消息内容
            json.dumps(data_dict),
            # 消息标签
            tag
        )
        # 这里sdk有点奇怪，第一次发送都会失败，需要发送两次。
        try:
            re_msg = self._get_producer(mq).publish_message(msg)
            if GB.global_server_log_func:
                GB.global_server_log_func(f"发送通知消息中间件完成 id:{re_msg.message_id} data:{data_dict}")
            return re_msg
        except MQExceptionBase as e:
            sleep(1)
            return self.send_message(mq, data_dict, tag)

    def get_message(self, max_time: int, mq, tag: str = "", auto_ack=False) -> Message:
        """

        :rtype: : Message
        """
        mq_client = MQClient(
            # 设置HTTP接入域名（此处以公共云生产环境为例）
            "http://1398845622246819.mqrest.cn-hangzhou.aliyuncs.com",
            # AccessKey 阿里云身份验证，在阿里云服务器管理控制台创建
            "LTAI4G2Nnc3tYNmkjGo5NAhd",
            # SecretKey 阿里云身份验证，在阿里云服务器管理控制台创建
            "dOfvhIja236buFblj0lAl1LVU8MNwf"
        )

        try:
            consumer = mq_client.get_consumer(mq.instanceId, mq.topic, mq.groupId, tag)
            recv_msgs = consumer.consume_message(1, 3)
            msg = recv_msgs[0]
            consumer.ack_message([msg.receipt_handle])
            return msg
        except Exception as e:
            if "MessageNotExist" not in str(e):
                print(e)
            return None

    def ack_message(self, mq, tag: str = "", handle=None):
        if handle is None:
            handle = []
        consumer = self._get_consumer(mq, tag)
        try:
            consumer.ack_message(handle)
        except MQExceptionBase as e:
            return
