# coding=utf-8
import hashlib
import json
import os
import subprocess
import time
import traceback
from time import sleep

import requests
from kthread import KThread

from bbcloud_autotest_sdk.airtest_babybus.core.android.adb import ADB


def get_phone_code(tel):
    s = requests.session()
    s.keep_alive = False
    url = 'http://udb.babybus.com/AppSync/GetPhoneCaptcha?Phone=%s&AccountGroupID=1' % tel
    r = s.get(url)

    str_txt = r.text.encode('utf-8').decode('utf-8')

    dic = json.loads(str_txt)

    result_code = dic['ResultCode']
    if result_code == '0':
        code = dic['Data']['CaptchaNo']
        return code
    return '000000'


def get_devices_sernum_list():
    return list(map(lambda x: x[0], ADB().devices('device')))


def download_file(url, save_path, retry=0):
    """
    下载文件
    :param retry:
    :param url: URL
    :param save_path: 保存文件名
    :return:
    """
    try:
        req = requests.get(url, timeout=60 * 10 * 1000)
        basename = os.path.dirname(save_path)
        if not os.path.exists(basename):
            os.makedirs(basename)
        with open(save_path, 'wb') as f:
            f.write(req.content)
    except Exception as e:
        if retry >= 5:
            raise Exception(e)
        download_file(url, save_path, retry + 1)


def md5(content):
    return hashlib.new('md5', content.encode('utf-8')).hexdigest()


def execute(command, cwd="/", buf_size=1024):
    """exec
    执行本地命令
    :param command: 命令
    :param cwd: 执行环境
    :param buf_size: 块大小
    :return: 输出，错误信息(bytes类型)
    """
    sub = subprocess.Popen(command, cwd=cwd, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE, shell=True, bufsize=buf_size)
    (stdout, stderr) = sub.communicate()
    return stdout, stderr


def get_mid_string(html, start_str, end):
    start = html.find(start_str)
    if start >= 0:
        start += len(start_str)
        end = html.find(end, start)
        if end >= 0:
            return html[start:end].strip()
    return ""


def get_mid_string_num(html, start_str, end):
    sub_str = get_mid_string(html, start_str, end)
    if sub_str == "":
        sub_str = "0"
    return int(sub_str)


# def get_thread_code_context(handle):
#     try:
#         info = UtilsProcess.get_thread_position(handle, "pages")
#         with open(info['file'], "r") as f:
#             lines = f.read().split('\n')
#             lines[info['line'] - 1] = lines[info['line'] - 1].replace(info['code'], "->" + info['code'])
#             codes = lines[(info['line'] - 3):(info['line'] + 3)]
#             redis_obj.set("phone_state_code_" + GB.global_adb_id, "\n".join(codes))
#     except:
#         pass


def my_timeout_check(timeout):
    def war(func):
        def inner(*args):
            handle = KThread(target=func, args=args)
            handle.setName("timeout thread" + str(func))
            handle.start()
            time_count = 0
            while time_count <= timeout:
                time_count += 1
                if not handle.is_alive():
                    return
                sleep(1)

            try:
                handle.kill()
            except:
                pass
            raise TimeoutError("函数运行超时，%s" % str(func))

        return inner

    return war


def case_report(desc):
    """
    重要修饰器，每个case都加，对case是否成功运行进行判断，同时记录结果
    :param desc: 中文描述
    :return:
    """

    def inner_function(input_func):
        def war(*args, **kwargs):
            pass

        # app_start_time = time.time()
        # name = str(input_func).split("function ")[1].split(" at")[0]
        #
        # def inner_imp():
        #     """
        #     线程内部的记录机制，用来记录任务是否正常完成
        #     """
        #     Report.get_instance().begin_test_case(name, desc)
        #     try:
        #         GB.global_task_handle_obj.send_runtime_log("运行测试用例:%s" % name)
        #         input_func(*args, **kwargs)
        #         Report.get_instance().end_test_case(True, None)
        #     except Exception as E:
        #         msg = traceback.format_exc()
        #         GB.global_task_handle_obj.send_runtime_log(msg)
        #         Report.get_instance().end_test_case(False, msg)
        #         GB.global_context['err'] = E
        #
        # try:
        #     handle = KThread(target=inner_imp, args=())
        #     handle.setName("script runner thread")
        #     handle.start()
        #     count = 0
        #     while handle.is_alive():
        #         count += 1
        #         sleep(1)
        #         get_thread_code_context(handle)
        #         if count > CASE_DEFAULT_TIME_OUT:
        #             handle.kill()
        #             GB.global_screen_shot_func("time out scene time out!")
        #             Report.get_instance().end_test_case(False, "运行测试用例超时")
        #             raise Exception("测试用例运行超时")
        #     if GB.global_context.get('err'):
        #         raise Exception(GB.global_context['err'])
        # except Exception as e:
        #     raise Exception(e)
        # return

        return war

    return inner_function


def reset_all_usb():
    from bbcloud_autotest_sdk.lib.adb import adb_commands, sign_cryptography

    signer = sign_cryptography.CryptographySigner(
        os.path.expanduser('~/.android/adbkey'))

    device = adb_commands.AdbCommands()

    devices_list = [item for item in device.Devices()]
    # devices_sernum_list = [v.serial_number for v in devices_list]
    alive_list = get_devices_sernum_list()
    for dev in devices_list:
        try:
            if dev.serial_number not in alive_list:
                print("设备 %s 没有正常工作，试图重置usb设备。" % dev.serial_number)
                text_array = str(dev._device).split(" ")
                v_num = text_array[5].split(':')[0]
                p_num = text_array[5].split(':')[1]
                print(v_num, p_num)
                os.system("usb_modeswitch -v " + v_num + " -p " + p_num + " --reset-usb")
        except Exception as e:
            print(e)
